//          _/             _/_/
//        _/_/           _/_/_/
//      _/_/_/_/         _/_/_/
//      _/_/_/_/_/       _/_/_/              ____________________________________________ 
//      _/_/_/_/_/       _/_/_/             /                                           / 
//      _/_/_/_/_/       _/_/_/            /                                   M25P128 / 
//      _/_/_/_/_/       _/_/_/           /                                           /  
//      _/_/_/_/_/_/     _/_/_/          /                                   128Mbit / 
//      _/_/_/_/_/_/     _/_/_/         /                              SERIAL FLASH / 
//      _/_/_/ _/_/_/    _/_/_/        /                                           / 
//      _/_/_/  _/_/_/   _/_/_/       /                  Verilog Behavioral Model / 
//      _/_/_/   _/_/_/  _/_/_/      /                               Version 1.0 / 
//      _/_/_/    _/_/_/ _/_/_/     /                                           /
//      _/_/_/     _/_/_/_/_/_/    /           Copyright (c) 2008 Numonyx B.V. / 
//      _/_/_/      _/_/_/_/_/    /___________________________________________/ 
//      _/_/_/       _/_/_/_/      
//      _/_/          _/_/_/  
// 
//     
//             NUMONYX              


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-----------------------------------------------------------
-----------------------------------------------------------
--                                                       --
--           PARAMETERS OF DEVICES                       --
--                                                       --
-----------------------------------------------------------
-----------------------------------------------------------
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/



//-----------------------------
//  Customization Parameters
//-----------------------------

`define timingChecks


//-- Available devices: M25PX32, M25PE16, M25P128
`define M25P128






//----------------------------
// Model configuration
//----------------------------


parameter dataDim = 8;


`ifdef M25PX32

    parameter [7*8:1] devName = "M25PX32";
    parameter addrDim = 22;
    parameter sectorAddrDim = 6;
    parameter [dataDim-1:0] MemoryType_ID = 'h63;
    parameter [dataDim-1:0] MemoryCapacity_ID = 'h16;
    `define HOLD_pin
    `define Vpp_pin
    `define OTP
    `define DUAL
    `define SubSect
    `define LockReg
    `define PowDown
    
`elsif M25PE16

    parameter [7*8:1] devName = "M25PE16";
    parameter addrDim = 21;
    parameter sectorAddrDim = 5;
    parameter [dataDim-1:0] MemoryType_ID = 'h80;
    parameter [dataDim-1:0] MemoryCapacity_ID = 'h15;
    `define W_pin
    `define RESET_pin
    `define SubSect
    `define LockReg
    `define PowDown

`elsif M25P128

    parameter [7*8:1] devName = "M25P128";
    parameter addrDim = 24;
    parameter sectorAddrDim = 6;
    parameter [dataDim-1:0] MemoryType_ID = 'h20;
    parameter [dataDim-1:0] MemoryCapacity_ID = 'h18;
    `define HOLD_pin
    `define Vpp_W_pin

`endif






//----------------------------
// Include TimingData file 
//----------------------------

`include "include/TimingData.h"





//----------------------------------------
// Parameters constants for all devices
//----------------------------------------

`define VoltageRange 31:0




//---------------------------
// stimuli clock period
//---------------------------
// for a correct behavior of the stimuli, clock period should
// be multiple of 4

`ifdef M25PX32
  parameter time T = 40;
`elsif M25PE16
  parameter time T = 40;
`elsif M25P128
  parameter time T = 52;
`endif  






//-----------------------------------
// Devices Parameters 
//-----------------------------------


// data & address dimensions

parameter cmdDim = 8;
parameter addrDimLatch = 24;


// memory organization

parameter memDim = 2 ** addrDim; 

parameter colAddrDim = 8;
parameter colAddr_sup = colAddrDim-1;
parameter pageDim = 2 ** colAddrDim;

parameter nSector = 2 ** sectorAddrDim;
`ifdef SubSect
 parameter sectorAddr_inf = 16;
`elsif M25P128
 parameter sectorAddr_inf = addrDim-sectorAddrDim; `endif

parameter sectorAddr_sup = addrDim-1;
parameter sectorSize = 2 ** (addrDim-sectorAddrDim);

`ifdef SubSect
 parameter subsecAddrDim = 4+sectorAddrDim;
 parameter subsecAddr_inf = 12;
 parameter subsecAddr_sup = addrDim-1;
 parameter subsecSize = 2 ** (addrDim-subsecAddrDim); `endif

parameter pageAddrDim = addrDim-colAddrDim;
parameter pageAddr_inf = colAddr_sup+1;
parameter pageAddr_sup = addrDim-1;




// OTP section

`ifdef OTP
 parameter OTP_dim = 65;
 parameter OTP_addrDim = 7;
`endif



//  signature 

parameter [dataDim-1:0] Manufacturer_ID = 'h20;



// others constants

parameter [dataDim-1:0] data_NP = 'hFF;



// voltage & startup timing constants

parameter [`VoltageRange] Vcc_wi = 'd2500; //write inhibit 
parameter [`VoltageRange] Vcc_min = 'd2700;
parameter [`VoltageRange] Vcc_max = 'd3600;
parameter [`VoltageRange] Vpp_min = 'd8500;
parameter [`VoltageRange] Vpp_max = 'd9500;





//-------------------------
// Alias used in the code
//-------------------------


// status register code

`define WIP M25Pxxx.stat.SR[0]

`define WEL M25Pxxx.stat.SR[1]

`define BP0 M25Pxxx.stat.SR[2]

`define BP1 M25Pxxx.stat.SR[3]

`define BP2 M25Pxxx.stat.SR[4]

`ifdef M25PX32
  `define TB M25Pxxx.stat.SR[5] `endif

`ifdef M25PX32
  `define LOTP M25Pxxx.stat.SR[7]
`elsif M25PE16 
  `define SRWD M25Pxxx.stat.SR[7]
`elsif M25P128 
  `define SRWD M25Pxxx.stat.SR[7]  
`endif



// input-output pin


`ifdef M25PX32
    `define D DQ0
    `define Q DQ1
`else 
    `define D D
    `define Q Q
`endif    






