//          _/             _/_/
//        _/_/           _/_/_/
//      _/_/_/_/         _/_/_/
//      _/_/_/_/_/       _/_/_/              ____________________________________________ 
//      _/_/_/_/_/       _/_/_/             /                                           / 
//      _/_/_/_/_/       _/_/_/            /                                   M25P128 / 
//      _/_/_/_/_/       _/_/_/           /                                           /  
//      _/_/_/_/_/_/     _/_/_/          /                                   128Mbit / 
//      _/_/_/_/_/_/     _/_/_/         /                              SERIAL FLASH / 
//      _/_/_/ _/_/_/    _/_/_/        /                                           / 
//      _/_/_/  _/_/_/   _/_/_/       /                  Verilog Behavioral Model / 
//      _/_/_/   _/_/_/  _/_/_/      /                               Version 1.0 / 
//      _/_/_/    _/_/_/ _/_/_/     /                                           /
//      _/_/_/     _/_/_/_/_/_/    /           Copyright (c) 2008 Numonyx B.V. / 
//      _/_/_/      _/_/_/_/_/    /___________________________________________/ 
//      _/_/_/       _/_/_/_/      
//      _/_/          _/_/_/  
// 
//     
//             NUMONYX              


/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-----------------------------------------------------------
-----------------------------------------------------------
--                                                       --
--           TIMING CONSTANTS                            --
--                                                       --
-----------------------------------------------------------
-----------------------------------------------------------
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/




`ifdef M25PX32
// From datasheet M25PX32 - December 2006 - Rev 0.1

    //--------------------------------------
    // Max clock frequency (minimum period)
    //--------------------------------------
    
    parameter fC = 75; //max frequency in Mhz
    parameter time TC = 13; //13ns=76.92Mhz
    
    parameter fR = 33; //max frequency in Mhz during standard Read operation
    parameter time TR = 30; //30ns=33.33Mhz

    //NB: in special read operations 
    // (fast read, dual output fast read, read SR, 
    // read lock register, read ID, read OTP)
    // max clock frequency is fC 
    
    //---------------------------
    // Input signals constraints
    //---------------------------

    // Clock signal constraints
    parameter time tCH = 6;
    parameter time tCL = 6;
    parameter time tSLCH = 5;
    parameter time tDVCH = 2;
    parameter time tSHCH = 5;
    parameter time tHLCH = 5;
    parameter time tHHCH = 5;

    // Chip select constraints
    parameter time tCHSL = 5;
    parameter time tCHSH = 5;
    parameter time tSHSL = 100;
    parameter time tVPPHSL = 200; //not implemented

    // Data in constraints
    parameter time tCHDX = 5;

    // HOLD signal constraints
    parameter time tCHHH = 5;
    parameter time tCHHL = 5;


    //-----------------------
    // Output signal timings
    //-----------------------

    parameter time tSHQZ = 8;
    parameter time tCLQV = 8; // 8 under 30 pF - 6 under 10 pF
    parameter time tCLQX = 0; // min value
    parameter time tHHQX = 8;  
    parameter time tHLQZ = 8;


    //--------------------
    // Operation delays
    //--------------------
    
    parameter time tDP  = 3e3;
    parameter time tRDP = 30e3; 
    parameter time tW   = 15e6;
    parameter time tPP  = 5e6;
    parameter time tSSE = 150e6;
    parameter time tSE  = 3e9;
    parameter time tBE  = 80e9;

    // Startup delays
    parameter time tPUW = 10e6;
    parameter time tVSL = 30e3;






`elsif M25PE16
// From datasheet M25PE16 - April 2007 - Rev 4



    //--------------------------------------
    // Max clock frequency (minimum period)
    //--------------------------------------
    
    parameter fC = 50; //max frequency in Mhz
    parameter time TC = 20;

    parameter fR = 33; //max frequency in Mhz during standard Read operation
    parameter time TR = 30; //30ns=33.33Mhz

    //NB: in special read operations 
    // (fast read, read SR, read lock register, read ID, read OTP)  
    // max clock frequency is fC
    

    //---------------------------
    // Input signals constraints
    //---------------------------

    // Clock signal constraints
    parameter time tCH = 9;
    parameter time tCL = 9;
    parameter time tSLCH = 5;
    parameter time tDVCH = 2;
    parameter time tSHCH = 5;

    // Chip select constraints
    parameter time tCHSL = 5;
    parameter time tCHSH = 5;
    parameter time tSHSL = 100;
    parameter time tRHSL_1 = 30e3; //during decoding
    parameter time tRHSL_2 = 300e3; //during program-erase operation (except subsector erase and WRSR)
    parameter time tRHSL_3 = 3e6;  //during subsector erase operation
    parameter time tRHSL_4 = 15e6; //during WRSR operation
    parameter time tRHSL_5 = 0; //when S is high
    parameter time tWHSL = 50; 
   
    // Data in constraints
    parameter time tCHDX = 5;

    // W signal constraints
    parameter time tSHWL = 100; 

    // RESET signal constraints
    parameter time tRLRH = 10e3; 
    parameter time tSHRH = 10; 


    //-----------------------
    // Output signal timings
    //-----------------------

    parameter time tSHQZ = 8;
    parameter time tCLQV = 8; 
    parameter time tCLQX = 0; //min value


    //--------------------
    // Operation delays
    //--------------------
    
    parameter time tDP  = 3e3;
    parameter time tRDP = 30e3; 
    parameter time tW   = 15e6;
    parameter time tPW  = 23e6;
    parameter time tPP  = 3e6;
    parameter time tPE  = 20e6;
    parameter time tSSE = 150e6;
    parameter time tSE  = 5e9;
    parameter time tBE  = 60e9;

    // Startup delays
    parameter time tPUW = 10e6;
    parameter time tVSL = 30e3;








`elsif M25P128 
// From datasheet: M25P128 - Rev 2 - October 2006 



    //--------------------------------------
    // Max clock frequency (minimum period)
    //--------------------------------------
    
    parameter fC = 50; //max frequency in Mhz
    parameter time TC = 20;

    parameter fR = 50; //max frequency in Mhz during standard Read operation
    parameter time TR = 50; 

    //NB: in special read operations 
    // (fast read, read SR, read lock register, read ID, read OTP)  
    // max clock frequency is fC
    

    //---------------------------
    // Input signals constraints
    //---------------------------

    // Clock signal constraints
    parameter time tCH = 9;
    parameter time tCL = 9;
    parameter time tSLCH = 5;
    parameter time tDVCH = 2;
    parameter time tSHCH = 5;
    parameter time tHLCH = 5;
    parameter time tHHCH = 5;

    // Chip select constraints
    parameter time tCHSL = 5;
    parameter time tCHSH = 5;
    parameter time tSHSL = 100;
    parameter time tWHSL = 20; 
   
    // Data in constraints
    parameter time tCHDX = 5;

    // W signal constraints
    parameter time tSHWL = 100; 

    // HOLD signal constraints
    parameter time tCHHH = 5;
    parameter time tCHHL = 5;


    //-----------------------
    // Output signal timings
    //-----------------------

    parameter time tSHQZ = 8;
    parameter time tCLQV = 8; 
    parameter time tCLQX = 0; //min value
    parameter time tHHQX = 8;  
    parameter time tHLQZ = 8;


    //--------------------
    // Operation delays
    //--------------------
    
    parameter time tW   = 15e6;
    parameter time tPP  = 7e6;
    parameter time tSE  = 6e9;
    parameter time tBE  = 250e9;

    // Startup delays
    parameter time tPUW = 10e6;
    parameter time tVSL = 60e3;




`endif
    








//---------------------------------
// Alias of timing constants above
//---------------------------------

parameter time program_delay = tPP;
parameter time write_SR_delay = tW;
parameter time erase_delay = tSE;
parameter time erase_bulk_delay = tBE;
parameter time write_access_power_up_delay = tPUW;
parameter time read_access_power_up_delay = tVSL;

`ifdef SubSect //M25PX32, M25PE16 
  parameter time erase_ss_delay = tSSE;
`endif

`ifdef PowDown //M25PX32, M25PE16
  parameter time deep_power_down_delay = tDP; 
  parameter time release_power_down_delay = tRDP;
`endif

`ifdef M25PE16
  parameter time page_write_delay = tPW;
  parameter time erase_page_delay = tPE;
`endif
