//          _/             _/_/
//        _/_/           _/_/_/
//      _/_/_/_/         _/_/_/
//      _/_/_/_/_/       _/_/_/              ____________________________________________ 
//      _/_/_/_/_/       _/_/_/             /                                           / 
//      _/_/_/_/_/       _/_/_/            /                                   M25P128 / 
//      _/_/_/_/_/       _/_/_/           /                                           /  
//      _/_/_/_/_/_/     _/_/_/          /                                   128Mbit / 
//      _/_/_/_/_/_/     _/_/_/         /                              SERIAL FLASH / 
//      _/_/_/ _/_/_/    _/_/_/        /                                           / 
//      _/_/_/  _/_/_/   _/_/_/       /                  Verilog Behavioral Model / 
//      _/_/_/   _/_/_/  _/_/_/      /                               Version 1.0 / 
//      _/_/_/    _/_/_/ _/_/_/     /                                           /
//      _/_/_/     _/_/_/_/_/_/    /           Copyright (c) 2008 Numonyx B.V. / 
//      _/_/_/      _/_/_/_/_/    /___________________________________________/ 
//      _/_/_/       _/_/_/_/      
//      _/_/          _/_/_/  
// 
//     
//             NUMONYX              


//-----------------------------
// For all the M25Pxxx devices
//-----------------------------



`include "top/StimGen_interface.h"
// the port list of current module is contained in "StimGen_interface.h" file


    parameter [12*8:1] mem = "mem_P128.vmf";    
    
    defparam Testbench.DUT.memory_file = mem;

    // transactions handles
    integer trans, fiber;


    reg [addrDim-1:0] S0; 
    reg [addrDim-1:0] S1; 
    
    reg [addrDim-1:0] addr='h0;   



    integer i;
    


    initial begin


        S0 = 'h200000;
        S1 = 'h23FFFE;
        
        
        fiber = $sdi_create_fiber("Test fiber");

        tasks.init;


        //---------------
        // Sector erase 
        //---------------

        $display("\n---- Sector erase");

        // sector erase 
        trans = $sdi_transaction("BeginEnd",fiber,"Sector erase");
        tasks.write_enable;
        tasks.send_command('hD8);
        tasks.send_address(S0); 
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #(erase_delay+100);

        // read 1 (begin of the sector) 
        trans = $sdi_transaction("BeginEnd",fiber,"Read 1");
        tasks.send_command('h03);
        tasks.send_address(S0);
        tasks.read(3);
        tasks.close_comm;
        $sdi_end_transaction(trans);

        // read 2 (end of the sector) 
        trans = $sdi_transaction("BeginEnd",fiber,"Read 2");
        tasks.send_command('h03);
        tasks.send_address(S1);
        tasks.read(4);
        tasks.close_comm;
        $sdi_end_transaction(trans);

       


        //-----------------
        // Bulk erase 
        //-----------------

        $display("\n---- Bulk erase");

        // bulk erase (ok) 
        trans = $sdi_transaction("BeginEnd",fiber,"Bulk erase");
        tasks.write_enable;
        tasks.send_command('hC7);
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #100;

        // read SR 
        trans = $sdi_transaction("BeginEnd",fiber,"Read SR");
        tasks.send_command('h05);
        tasks.read(2);
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #erase_bulk_delay;
        
        // read  
        trans = $sdi_transaction("BeginEnd",fiber,"Read");
        tasks.send_command('h03);
        tasks.send_address(addr);
        tasks.read(3);
        tasks.close_comm;
        $sdi_end_transaction(trans);
        


    end


endmodule    
