//          _/             _/_/
//        _/_/           _/_/_/
//      _/_/_/_/         _/_/_/
//      _/_/_/_/_/       _/_/_/              ____________________________________________ 
//      _/_/_/_/_/       _/_/_/             /                                           / 
//      _/_/_/_/_/       _/_/_/            /                                   M25P128 / 
//      _/_/_/_/_/       _/_/_/           /                                           /  
//      _/_/_/_/_/_/     _/_/_/          /                                   128Mbit / 
//      _/_/_/_/_/_/     _/_/_/         /                              SERIAL FLASH / 
//      _/_/_/ _/_/_/    _/_/_/        /                                           / 
//      _/_/_/  _/_/_/   _/_/_/       /                  Verilog Behavioral Model / 
//      _/_/_/   _/_/_/  _/_/_/      /                               Version 1.0 / 
//      _/_/_/    _/_/_/ _/_/_/     /                                           /
//      _/_/_/     _/_/_/_/_/_/    /           Copyright (c) 2008 Numonyx B.V. / 
//      _/_/_/      _/_/_/_/_/    /___________________________________________/ 
//      _/_/_/       _/_/_/_/      
//      _/_/          _/_/_/  
// 
//     
//             NUMONYX              



`include "top/StimGen_interface.h"
// the port list of current module is contained in "StimGen_interface.h" file


    defparam Testbench.DUT.memory_file = "";

    // transactions handles
    integer trans, fiber;


    reg [addrDim-1:0] A1, A2;
    
    reg [dataDim-1:0] regData = 'h0;

    integer i;
    


    initial begin


        A1='hE80000; //sector 58 (unlocked)
        A2='hFC0000; //sector 63 (locked by SR) 
        
        fiber = $sdi_create_fiber("Test fiber");

        tasks.init;

        //--------------------------------------------------------
        // Testing locking features controlled by status register
        //--------------------------------------------------------

        // write status register 
        $display("\n--- Write status register");
        trans = $sdi_transaction("BeginEnd",fiber,"Write status register");
        tasks.write_enable;
        tasks.send_command('h01);
        regData[4:2] = 'b011; // (BP2,BP1,BP0)=(0,1,1) - Lock sector: 60-63
        tasks.send_data(regData); 
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #(write_SR_delay+100);  

        // page program A2 in sector locked (error) (status register lock enabled)
        $display("\n--- Program error");
        trans = $sdi_transaction("BeginEnd",fiber,"Program error");
        tasks.write_enable;
        tasks.send_command('h02);
        tasks.send_address(A2);
        tasks.send_data('hA2);
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #100;

        // page program A2 in sector unlocked (ok) (status register lock disabled)
        $display("\n--- Program ok");
        trans = $sdi_transaction("BeginEnd",fiber,"Program ok");
        tasks.write_enable;
        tasks.send_command('h02);
        tasks.send_address(A1);
        tasks.send_data('hA1);
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #(100+program_delay);

        // write status register (set SRWD bit) 
        $display("\n--- write status register (set SRWD bit)");
        trans = $sdi_transaction("BeginEnd",fiber,"Write status register");
        tasks.write_enable;
        tasks.send_command('h01);
        regData[4:2] = 'b011; 
        regData[7] = 1'b1; // SRWD bit
        tasks.send_data(regData); 
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #(write_SR_delay+100);

        // write status register (change is possible: SRWD bit is set, but W is high)
        $display("\n--- write status register (change is possible because W is high)");
        trans = $sdi_transaction("BeginEnd",fiber,"Write status register");
        tasks.write_enable;
        tasks.send_command('h01);
        regData[4:2] = 'b100;  
        tasks.send_data(regData); 
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #(write_SR_delay+100);

        tasks.set_Vpp_W(0);

        // write status register (error because SRWD bit is set and W is low) 
        $display("\n--- write status register (error because SRWD bit is set and W is low)");
        trans = $sdi_transaction("BeginEnd",fiber,"Write status register");
        tasks.write_enable;
        tasks.send_command('h01);
        regData[4:2] = 'b000;  
        tasks.send_data(regData); 
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #100;

        // read status register 
        $display("\n--- read status register");
        trans = $sdi_transaction("BeginEnd",fiber,"Read status register");
        tasks.send_command('h05);
        tasks.read(2); 
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #100;
        

    end


endmodule    
