//          _/             _/_/
//        _/_/           _/_/_/
//      _/_/_/_/         _/_/_/
//      _/_/_/_/_/       _/_/_/              ____________________________________________ 
//      _/_/_/_/_/       _/_/_/             /                                           / 
//      _/_/_/_/_/       _/_/_/            /                                   M25P128 / 
//      _/_/_/_/_/       _/_/_/           /                                           /  
//      _/_/_/_/_/_/     _/_/_/          /                                   128Mbit / 
//      _/_/_/_/_/_/     _/_/_/         /                              SERIAL FLASH / 
//      _/_/_/ _/_/_/    _/_/_/        /                                           / 
//      _/_/_/  _/_/_/   _/_/_/       /                  Verilog Behavioral Model / 
//      _/_/_/   _/_/_/  _/_/_/      /                               Version 1.0 / 
//      _/_/_/    _/_/_/ _/_/_/     /                                           /
//      _/_/_/     _/_/_/_/_/_/    /           Copyright (c) 2008 Numonyx B.V. / 
//      _/_/_/      _/_/_/_/_/    /___________________________________________/ 
//      _/_/_/       _/_/_/_/      
//      _/_/          _/_/_/  
// 
//     
//             NUMONYX              

//-----------------------------
// For all the M25Pxxx devices
//-----------------------------



`include "top/StimGen_interface.h"
// the port list of current module is contained in "StimGen_interface.h" file 


    defparam Testbench.DUT.memory_file = "mem_P128.vmf"; 

    // transactions handles
    integer trans, fiber;

    reg [addrDim-1:0] A, B, Z, A0='h0, A1='h08;


    initial begin


        A='h200000; //sector 8
        B='h23FFFF; //end of sector 8
        Z='hFFFFFA; //end of memory

        fiber = $sdi_create_fiber("Test fiber");
        
        tasks.init;

        //----------------
        //  Standard read
        //----------------
        $display("\n ----- Read.");

        // read from memory file
        trans = $sdi_transaction("BeginEnd", fiber, "Read 1");
        tasks.send_command('h03);
        tasks.send_address(A);
        tasks.read(6);
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #100;

        // read from memory file
        trans = $sdi_transaction("BeginEnd", fiber, "Read 2");
        tasks.send_command('h03);
        tasks.send_address(B);
        tasks.read(2);
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #100;

        // read from memory file
        trans = $sdi_transaction("BeginEnd", fiber, "Read 3");
        tasks.send_command('h03);
        tasks.send_address(Z);
        tasks.read(8);
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #100;


        //----------------
        //  Fast read
        //----------------
        $display("\n ----- Fast Read.");
        
        // fast read from memory file
        trans = $sdi_transaction("BeginEnd", fiber, "Fast Read");
        tasks.send_command('h0B);
        tasks.send_address(A0);
        tasks.send_data('h00); //dummy byte
        tasks.read(4);
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #100;


        //----------------
        //  Read ID
        //----------------
        $display("\n ----- Read ID.");
        
        // read ID
        trans = $sdi_transaction("BeginEnd", fiber, "Read ID");
        tasks.send_command('h9F);
        tasks.read(8);
        tasks.close_comm;
        $sdi_end_transaction(trans);
        #100;

        
    end


endmodule    
