`timescale 1ns/1ns

module ApplicationFpga(

    input   SysAppClk_ik,
    output  SysAppClk_ok,
    inout   [2:1] SysAppSlow_iob2,

    inout   [7:0] AFpgaProgD_iob8, 
    inout   AFpgaProgClk_io,
    inout   [1:0] AFpgaProgM_iob2,
    inout   AFpgaProgCsi_io,
    inout   AFpgaProgRdWr_io,
    inout   AFpgaProgInit_io,

    inout   [4:1] FpGpIo_iob4,

    input   PushButton_ion);

`define dly #1

//#####################################
//  Clock
//##################################### 

wire Clk_k =  SysAppClk_ik;


//#####################################
//  Reset Signal Generation
//#####################################

wire DeboucedPushButton_q;
Debouncer #(.g_CounterWidth(16), .g_SynchDepth(3)) i_Debouncer (
    .Clk_ik(Clk_k),
    .BouncingSignal_ia(PushButton_ion),
    .DebouncedSignal_oq(DeboucedPushButton_q));

reg Rst_rq;
always @(posedge Clk_k) Rst_rq <= `dly ~DeboucedPushButton_q || RstWB;
        

//#####################################
//  Front Panel Signals
//#####################################  

assign FpGpIo_iob4[4] = Clk_k; 
assign FpGpIo_iob4[3] = FpGpIo_iob4[1];
assign FpGpIo_iob4[2] = Clk_k; 

//#####################################
//  WishBone Serial Interface
//#####################################

wire [20:0] Adr_b21;
wire Cyc, We, StbMaster, AckMaster;
wire [31:0] DatMasterO_b32, DatMasterI_b32;

Ser2MstWB i_Ser2MstWB(
    .Rst_orq(RstWB),
    .Clk_ik(Clk_k),
    .Cyc_o(Cyc),
    .We_o(We),
    .Adr_ob21(Adr_b21),
    .Dat_ob32(DatMasterO_b32),
    .Stb_o(StbMaster),
    .Dat_ib32(DatMasterI_b32),
    .Ack_i(AckMaster),
    .SerClk_ik(SysAppClk_ik),
    .SerDat_i(SysAppSlow_iob2[1]),
    .SerCntrl_i(SysAppSlow_iob2[2]), 
    .Stb_i(AFpgaProgD_iob8[7]),
    .SerClk_ok(SysAppClk_ok),
    .SerDat_o(AFpgaProgD_iob8[5]),
    .Ack_o(AFpgaProgD_iob8[4])); 


AddressDecoderWBApp i_AddressDecoderWB(
    .Adr_ib21(Adr_b21),
    .Stb_i(StbMaster),
    .Dat_ob32(DatMasterI_b32),
    .Ack_o(AckMaster),

    .DatDebugRegs_ib32(DatDebugRegsrO_b32),
    .AckDebugRegs_i(AckDebugRegs),
    .StbDebugRegs_o(StbDebugRegs));  

//#####################################
//  Debug Registers
//#####################################

wire [31:0] DebugReg0, DebugReg1, DebugReg2, DebugReg3;

wire [31:0] DatDebugRegsrO_b32;
wire StbDebugRegs, AckDebugRegs;  

Generic4OutputRegs i_DebugRegs(
    .Rst_irq(Rst_rq),
    .Clk_ik(Clk_k),
    .Cyc_i(Cyc),
    .Stb_i(StbDebugRegs),
    .We_i(We),
    .Adr_ib2(Adr_b21[1:0]),
    .Dat_ib32(DatMasterO_b32),
    .Dat_oab32(DatDebugRegsrO_b32),
    .Ack_oa(AckDebugRegs),  
    .Reg0Value_ob32(DebugReg0),
    .Reg1Value_ob32(DebugReg1),
    .Reg2Value_ob32(DebugReg2),
    .Reg3Value_ob32(DebugReg3));  

endmodule
