`timescale 1ns/1ns

module Debouncer (
    input   Clk_ik,
    input   BouncingSignal_ia,
    output reg DebouncedSignal_oq);

parameter g_CounterWidth    = 16;   // Bounching time < 2**g_CounterWidth*Period-of-Clk_ik
parameter g_SynchDepth      = 3;    // Number of FF in the synchronization chain (min is 2)

reg [g_CounterWidth-1:0] Counter_c; 
reg [g_SynchDepth-1:0] BouncingSignal_x;

`define dly #1

always @(posedge Clk_ik) BouncingSignal_x <= `dly {BouncingSignal_x[g_SynchDepth-2:0], BouncingSignal_ia};

wire a_BouncingSignal_q = BouncingSignal_x[g_SynchDepth-1];


`define s_Idle 1'b0
`define s_Count 1'b1

reg State_q, NextState_a;

always @(posedge Clk_ik) State_q <= `dly NextState_a;

always @* begin
    NextState_a = State_q;
    case (State_q)
        `s_Idle:    if (a_BouncingSignal_q != DebouncedSignal_oq) NextState_a    = `s_Count;
        `s_Count:  if (a_BouncingSignal_q == DebouncedSignal_oq) NextState_a    = `s_Idle;
        default:    NextState_a = `s_Idle;
    endcase
end

always @(posedge Clk_ik) 
    if (State_q == `s_Count) Counter_c <= `dly Counter_c + 1'b1;
    else Counter_c <= `dly 'b0;

always @(posedge Clk_ik) if (State_q == `s_Count && &Counter_c) DebouncedSignal_oq <= `dly a_BouncingSignal_q;

initial DebouncedSignal_oq = 1'b1;

endmodule
