`timescale 1 ns/ 1 ns

module InterruptManagerWB (
	input int_enable,

    input   Rst_irq,
    input   Clk_ik,
    input   Cyc_i,
    input   Stb_i,
    input   We_i,
    input   [1:0] Adr_ib2,
    input   [31:0] Dat_ib32,
    output  reg [31:0] Dat_oab32,
    output  Ack_oa,  
	
	input [7:0] interrupt_in,
	output reg osc_clk,
	input int_acknowledged,
	input clear_all,
	output assert_interrupt);
	
parameter dly	= 1;

parameter int_reg_addr			= 2'b00;
parameter mask_reg_addr			= 2'b01;
parameter fpga_status_reg_addr	= 2'b10;   
parameter new_int_mode_addr		= 2'b11;   
	
reg [7:0] mask_reg, int_masked_old; //if a 1 is in position i than the i_th	interrupt is masked
wire [7:0] int_masked, interrupt;
reg [2:0] int_pointer_r, int_pointer_w;
wire clear_int;
reg [7:0] int_fifo [7:0];
reg fifo_full, fifo_empty;
wire asynch_clk_change;
reg hs_int_mode;

wire int_in_fifo;
reg rora_roak;
reg ready4int;


reg [3:0] int_counter;

reg Stb_d;

always  @(posedge Clk_ik) 
    if (Rst_irq) Stb_d  <= # dly 1'b0;
    else Stb_d  <= # dly Stb_i;

wire NewStb_a = Stb_i && ~Stb_d;    

assign clear_int	= Cyc_i && ~We_i && NewStb_a && Adr_ib2==int_reg_addr;

always @(posedge Clk_ik) begin
	if (Rst_irq) begin
		{hs_int_mode, ready4int, rora_roak, mask_reg}	<= # dly 11'h2ff;
	end else if (Cyc_i && We_i && Stb_i && Adr_ib2==mask_reg_addr) begin
		{hs_int_mode, ready4int, rora_roak, mask_reg}	<= # dly Dat_ib32[10:0];
	end else if (hs_int_mode && ((rora_roak && Cyc_i && ~We_i && Stb_i && Adr_ib2==int_reg_addr) || (~rora_roak && int_acknowledged))) begin
		ready4int	<= # dly 1'b0;
	end
end

assign asynch_clk_change = ~Rst_irq && ~interrupt_in[6];

initial osc_clk = 1;

always @(posedge Clk_ik or negedge asynch_clk_change) begin
	if (~asynch_clk_change) begin
		osc_clk	<= # dly 1'b1;
	end else if (Cyc_i && We_i && Stb_i && Adr_ib2==mask_reg_addr) begin
		osc_clk	<= # dly Dat_ib32[31];
	end
end
	
assign int_masked	= interrupt_in	& ~mask_reg;

always @(posedge Clk_ik) begin
	if (Rst_irq) begin
		int_masked_old	<= # dly 8'hff;
	end else if (~int_enable) begin
		int_masked_old	<= # dly 8'hff;
	end else begin
		int_masked_old	<= # dly int_masked;
	end
end

assign interrupt		= int_masked & ~int_masked_old;
assign new_interrupt	= |interrupt;

always @(posedge Clk_ik) begin
	if (Rst_irq) begin
		int_counter	<= # dly 4'h0;
	end else if (~int_enable || clear_all) begin
		int_counter	<= # dly 4'h0;
	end else begin
		if (new_interrupt && ~fifo_full && ~int_acknowledged && ~&int_counter) begin
			int_counter	<= # dly int_counter + 1'b1;
		end else if (int_acknowledged && ~(new_interrupt && ~fifo_full) && |int_counter) begin
			int_counter	<= # dly int_counter - 1'b1;
		end
	end
end			

always @(posedge Clk_ik) begin
	if (Rst_irq) begin
		int_pointer_r	<= # dly 3'h0;
		int_pointer_w	<= # dly 3'h0;
	end else if (~int_enable || clear_all) begin
		int_pointer_r	<= # dly 3'h0;
		int_pointer_w	<= # dly 3'h0;
	end else begin
		if (new_interrupt && ~fifo_full) begin
			int_pointer_w	<= # dly int_pointer_w + 1'b1;
		end
		if (clear_int && ~fifo_empty) begin
			int_pointer_r	<= # dly int_pointer_r + 1'b1;
		end
	end
end


always @(posedge Clk_ik) begin
	if (Rst_irq) begin
		fifo_full	<= # dly 1'b0;
		fifo_empty	<= # dly 1'b1;
	end else if (~int_enable || clear_all) begin
		fifo_full	<= # dly 1'b0;
		fifo_empty	<= # dly 1'b1;
	end else begin
		if ((int_pointer_w+1'b1)==int_pointer_r && new_interrupt && ~clear_int) begin
			fifo_full	<= # dly 1'b1;
		end else if (int_pointer_w!=int_pointer_r) begin
			fifo_full	<= # dly 1'b0;
		end
		if ((int_pointer_r+1'b1)==int_pointer_w && clear_int && ~new_interrupt) begin
			fifo_empty	<= # dly 1'b1;
		end else if (int_pointer_w!=int_pointer_r) begin
			fifo_empty	<= # dly 1'b0;
		end
	end
end

assign int_in_fifo	= ~fifo_empty;

always @(posedge Clk_ik) begin
	if (new_interrupt && ~fifo_full) begin
		int_fifo[int_pointer_w]	<= # dly interrupt;
	end 
end

assign assert_interrupt = int_enable && ready4int && ((|int_counter && ~rora_roak) || (int_in_fifo && rora_roak));

wire [7:0] fpga_version;
wire [7:0] fpga_revision;
wire [15:0] date;

assign fpga_version		= 8'h00;  
assign fpga_revision	= 8'h01;
assign date				= {8'd6, 4'd10,4'd10};

assign Ack_oa = Stb_i&&Cyc_i;

always @* case(Adr_ib2)
	int_reg_addr		: Dat_oab32 <= # dly fifo_empty ? 32'h0000_0000 : {24'h0, int_fifo[int_pointer_r]};
	mask_reg_addr		: Dat_oab32 <= # dly {osc_clk, int_enable, 2'h0, 1'b0, int_pointer_r, 1'b0, int_pointer_w, int_counter, 5'h0, hs_int_mode, ready4int, rora_roak, mask_reg}; 
	fpga_status_reg_addr: Dat_oab32	<= # dly {date, fpga_version, fpga_revision};			
	default: Dat_oab32  <= # dly 32'hdead_beef;
	endcase

endmodule