module Ser2MstWB (
    output  Rst_orq,
    output  Clk_ik,
    output  reg Cyc_o,
    output  reg We_o,
    output  reg [20:0] Adr_ob21,
    output  reg [31:0] Dat_ob32,
    output  Stb_o,
    input   [31:0] Dat_ib32,
    input   Ack_i,

    input   SerClk_ik,
    input   SerDat_i,
    input   SerCntrl_i, //We, Cyc and Adr Rst are serialized on this line
    input   Stb_i,
    output  SerClk_ok,
    output  SerDat_o,
    output  reg Ack_o);

`define dly #1 

reg [31:0]  SerDatIShReg_b32,
            SerCntrlIShReg_b32;

reg [2:0]   StbI_d3;

always @(posedge SerClk_ik) StbI_d3 <= `dly {StbI_d3[1:0], Stb_i};

wire NewStbI_a =   StbI_d3[2:1]==2'b01;

always @(posedge SerClk_ik) SerDatIShReg_b32 <= `dly {SerDat_i, SerDatIShReg_b32[31:1]};
always @(posedge SerClk_ik) if (NewStbI_a) Dat_ob32 <= `dly SerDatIShReg_b32;

always @(posedge SerClk_ik) SerCntrlIShReg_b32        <= `dly {SerCntrl_i, SerCntrlIShReg_b32[31:1]};
always @(posedge SerClk_ik) if (NewStbI_a)  Cyc_o     <= `dly SerCntrlIShReg_b32[30];
always @(posedge SerClk_ik) if (NewStbI_a)  We_o      <= `dly SerCntrlIShReg_b32[29];
always @(posedge SerClk_ik) if (NewStbI_a)  Adr_ob21  <= `dly SerCntrlIShReg_b32[20:0];

reg [2:0] Rst_xb3;
always @(posedge SerClk_ik or posedge Rst_orq) 
    if (Rst_orq)        Rst_xb3[0] <= `dly 1'b0; 
    else if (NewStbI_a) Rst_xb3[0] <= `dly SerCntrlIShReg_b32[31];
always @(posedge Clk_ik) Rst_xb3[2:1] <= `dly Rst_xb3[1:0];
assign Rst_orq = Rst_xb3[2]; 
    
reg [1:0] StbI_xb2;

always @(posedge Clk_ik) begin
    if (Rst_orq) StbI_xb2 <= `dly 2'b0;
    else StbI_xb2 <= `dly {StbI_xb2[0], StbI_d3[1]};
end

assign  Stb_o =  StbI_xb2[1];

reg [30:0] AckI_d31;

always @(posedge Clk_ik) 
    if (Rst_orq) AckI_d31 <= `dly 'b0;
    else AckI_d31 <= `dly {AckI_d31[29:0], Ack_i};

always @(posedge Clk_ik)
    if (Rst_orq)  Ack_o <= `dly 1'b0;
    else Ack_o <= `dly AckI_d31[30] && Ack_i; 

wire NewAckI_a = ~AckI_d31[1] && Ack_i;

reg [31:0] DatInShReg_b32;

always @(posedge Clk_ik) 
    if (NewAckI_a) DatInShReg_b32 <= `dly Dat_ib32;
    else  DatInShReg_b32 <= `dly {1'b0, DatInShReg_b32[31:1]};

assign SerDat_o     = DatInShReg_b32[0];
assign SerClk_ok    = ~Clk_ik;

endmodule
