module Slv2SerWB (
    input   Rst_irq,
    input   Clk_ik,
    input   Cyc_i,
    input   We_i,
    input   [20:0] Adr_ib21,
    input   [31:0] Dat_ib32,
    input   Stb_i,
    output  reg [31:0] Dat_ob32,
    output  reg Ack_o,

    output  SerClk_ok,
    output  SerDat_o,
    output  SerCntrl_o, //We, Cyc and Adr Rst are serialized on this line
    output  Stb_o,
    input   SerClk_ik,
    input   SerDat_i,
    input   Ack_i);


`define dly #1 

reg [31:0]  DatOutShReg_b32 = 32'h0,
            DatInShReg_b32  = 32'h0,
            CntrlShReg_b32  = 32'h2;
reg [31:0]  StbShReg_b32    = 32'h2;

reg StbI_d;

always @(posedge Clk_ik) 
    if (Rst_irq) begin
        StbI_d  <= `dly 1'b0;
    end else begin
        StbI_d  <= `dly Stb_i; 
    end

wire NewStbI_a  = Stb_i && ~StbI_d;

always @(posedge Clk_ik) begin
    DatOutShReg_b32 <= `dly {1'b0, DatOutShReg_b32[31:1]};
    CntrlShReg_b32  <= `dly {1'b0, CntrlShReg_b32[31:1]};
    StbShReg_b32    <= `dly {1'b0, Stb_i, StbShReg_b32[30:1]};    
    if (Rst_irq) begin
        DatOutShReg_b32 <= `dly 32'h0;
        CntrlShReg_b32  <= `dly 32'h4;  
        StbShReg_b32    <= `dly 'h2;
    end else if (NewStbI_a) begin
        DatOutShReg_b32 <= `dly Dat_ib32;
        CntrlShReg_b32   <= `dly {Rst_irq, Cyc_i, We_i, 8'b0, Adr_ib21}; //Rst is useless here, but is just for documentation that i keep it
    end
end

assign SerDat_o     = DatOutShReg_b32[0];
assign SerCntrl_o   = CntrlShReg_b32[0];
assign Stb_o        = StbShReg_b32[0];
assign SerClk_ok    = ~Clk_ik;

reg [2:0] AckI_d3;

always @(posedge SerClk_ik) 
   if (Rst_irq) begin
       AckI_d3  <= `dly 'b0;
   end else begin
       AckI_d3  <= `dly {AckI_d3[1:0], Ack_i};
   end  

wire NewAckI_a  = AckI_d3[2:1]==2'b01;

reg [31:0] Dat_xb32;
always @(posedge SerClk_ik) DatInShReg_b32   <= `dly {SerDat_i, DatInShReg_b32[31:1]};
always @(posedge SerClk_ik) if (NewAckI_a) Dat_xb32 <= `dly  DatInShReg_b32;

reg [2:0] AckI_xb3;

always @(posedge SerClk_ik) AckI_xb3[2] <= `dly Ack_i;

always @(posedge Clk_ik) AckI_xb3[1:0]<= `dly AckI_xb3[2:1];


always @(posedge Clk_ik) if (AckI_xb3[0])  Dat_ob32 <= `dly Dat_xb32;

always @(posedge Clk_ik) Ack_o <= `dly AckI_xb3[0];

endmodule                                         