`timescale 1ns/1ns

module VmeInterfaceWB(

    input   rst_i,
    input   clk_i,
    output  reg [21:0] adr_o,
    output  reg [31:0] dat_o,
    input   [31:0] dat_i,
    output  reg we_o,
    output  reg stb_o,
    input   ack_i,   
    output  reg cyc_o,

    input UseGa_i,
    input [4:0] ManualAddress_i,  

	input [4:0] vme_ga,
	input vme_gap,
	input vme_as, 
	input vme_ds1,
	input vme_ds2,
	input [5:0] vme_am,
	input vme_wr,
	output reg vme_dtack,
	input vme_lword,
	input [31:1] vme_addr,
	inout [31:0] vme_data,
    output reg VmeDOe_o,
    output reg VmeDDirFpgaToVme_o,
	input 	vme_iackinn,
	input 	vme_iackn,
	output 	reg vme_iack_outn,
	output 	reg [7:1] vme_irqn,
	
	input 	[2:0] intlev_reg,
	input	[7:0] IrqVector_i,
	
	input assert_interrupt,
	output reg clear_int);
	
parameter dly = 1;

reg oe_vme_data;

reg [1:0] ds1_shr, ds2_shr;
reg [1:0] as_shr;
wire as_int, pre_as_int, ds_int;

wire [4:0] base_addr;
wire gap_error;
wire selected;

wire valid_am;
	
reg [2:0] state;

parameter s_idle	= 3'h0;
parameter s_read	= 3'h1;
parameter s_write	= 3'h2; 
parameter s_ack_int	= 3'h3;

assign gap_error	= ^{vme_ga, ~vme_gap}; 
assign base_addr	= (gap_error | ~UseGa_i)	? ManualAddress_i : ~vme_ga;

assign valid_am	= (vme_am==6'h0F) || (vme_am==6'h0D) || (vme_am==6'h0B) || (vme_am==6'h09);

reg [7:0] VmeBaseAddr;

always @(posedge clk_i) 
    if (rst_i) VmeBaseAddr  <= # dly 'h0;
    else if (as_int && ~pre_as_int) VmeBaseAddr <= # dly vme_addr[31:24];	
	

assign selected		= vme_iackn && ~as_int &&  ~ds_int && ~vme_lword && valid_am && (VmeBaseAddr=={3'b0, base_addr});

//******************************
//ADD OF A DEFAULT REPLAY IN CASE OF MISSING ANSWER FROM THE EXTERNAL MODULES
//******************************

reg [8:0] AckTimeout_c;
reg ack_d, stb_d;
reg [31:0] DataReg;

always @(posedge clk_i) ack_d <= # dly rst_i ? 1'b0 : ack_i;
always @(posedge clk_i) stb_d <= # dly rst_i ? 1'b0 : stb_o;

wire NewAck_a = ack_i && ~ack_d;
wire NewStb_i = stb_o && ~stb_d; 

always @(posedge clk_i) begin
	if (rst_i) begin
		AckTimeout_c	<= # dly 'h0;
	end else begin
		if (NewAck_a) begin
			AckTimeout_c	<= # dly 'h0;
		end else if (NewStb_i) begin
			AckTimeout_c	<= # dly 'h1;
		end else if (|AckTimeout_c) begin
			AckTimeout_c	<= # dly AckTimeout_c + 1'b1;
		end
	end
end
			
always @(posedge clk_i) if (NewAck_a || &AckTimeout_c) DataReg	<= # dly &AckTimeout_c ? 32'hdead_beef : dat_i;

//******************************

reg SendIrqVector;
assign vme_data	= oe_vme_data	? (SendIrqVector ? {24'h0, IrqVector_i} : DataReg) : 32'hz;

assign pre_as_int	= as_shr[0];
assign as_int		= as_shr[1];
assign ds_int		= ds1_shr[1] && ds2_shr[1];

always @(posedge clk_i) begin
	as_shr	<= # dly {as_shr[0], vme_as};
	ds1_shr	<= # dly {ds1_shr[0], vme_ds1};
	ds2_shr	<= # dly {ds2_shr[0], vme_ds2};
end		

always @(posedge clk_i) begin
	if (rst_i) begin
		vme_irqn	<= # dly 7'h7f;
	end else begin
		vme_irqn	<= # dly 7'h7f;
		if (assert_interrupt) begin
			vme_irqn[intlev_reg]	<= # dly 1'b0;
	 	end
	end
end

always @(posedge clk_i) begin
	if (rst_i) begin
		state			    <= # dly s_idle;
		oe_vme_data		    <= # dly 1'b0;
		vme_dtack		    <= # dly 1'b1;
		vme_iack_outn	    <= # dly 1'b1;
		clear_int		    <= # dly 1'b0;	
		VmeDOe_o 	        <= # dly 1'b0;
		VmeDDirFpgaToVme_o	<= # dly 1'b1;
        SendIrqVector       <= # dly 1'b0;  
        adr_o               <= # dly 22'h0;
        dat_o               <= # dly 32'h0;
        we_o                <= # dly 1'b0;
        stb_o               <= # dly 1'b0;
        cyc_o               <= # dly 1'b0;
	end else case (state)
		s_idle: begin
			state			    <= # dly s_idle;
            dat_o               <= # dly 32'h0;
            we_o                <= # dly 1'b0;
            stb_o               <= # dly 1'b0;
            cyc_o               <= # dly 1'b0;    		
			oe_vme_data		    <= # dly 1'b0;
			VmeDOe_o    	    <= # dly 1'b0;
			VmeDDirFpgaToVme_o  <= # dly 1'b1;
			vme_dtack		    <= # dly 1'b1;
			vme_iack_outn	    <= # dly 1'b1;
			clear_int		    <= # dly 1'b0;
			SendIrqVector	    <= # dly 1'b0;						
			if (as_int && ~pre_as_int) begin
				adr_o	<= # dly vme_addr[23:2];	
			end
			if (selected && ~ack_i) begin
				VmeDOe_o	<= # dly 1'b1;
				if (vme_wr) begin
					state	<= # dly s_read;
                    stb_o   <= # dly 1'b1;
                    cyc_o   <= # dly 1'b1;  
				end else begin
					state	            <= # dly s_write;
        			VmeDDirFpgaToVme_o	<= # dly 1'b0;
                    we_o                <= # dly 1'b1;
                    cyc_o               <= # dly 1'b1;  
				end
			end else if (~vme_iackinn) begin
				if (~as_int && vme_addr[3:1]==intlev_reg && ~&vme_irqn && ~ds1_shr[1]) begin
					state			<= # dly s_ack_int;
					clear_int		<= # dly 1'b1;			
					SendIrqVector	<= # dly 1'b1;	
		        	oe_vme_data		<= # dly 1'b1;
		    	    VmeDOe_o    	<= # dly 1'b1;
				end else if (~as_int) begin
					vme_iack_outn	<= # dly 1'b0;					
				end		
			end			
		end
		s_read: begin
			oe_vme_data		<= # dly 1'b1;
			if (ack_i || &AckTimeout_c) begin
				vme_dtack	<= # dly 1'b0;
                stb_o       <= # dly 1'b0;
                cyc_o       <= # dly 1'b0;
            end
			if (~selected) begin
				state		<= # dly s_idle;
				VmeDOe_o	<= # dly 1'b0;
				oe_vme_data	<= # dly 1'b0;
				adr_o       <= # dly adr_o + 1'b1;					
			end 
		end
		s_write: begin
			if (~selected) begin
				VmeDOe_o    	    <= # dly 1'b0;
                VmeDDirFpgaToVme_o  <= # dly 1'b1;
                state	            <= # dly s_idle;
				adr_o	            <= # dly adr_o + 1'b1;
            end
            if (ack_i || &AckTimeout_c) begin
                cyc_o       <= # dly 1'b0;
                we_o        <= # dly 1'b0;
                stb_o       <= # dly 1'b0;
				vme_dtack	<= # dly 1'b0;
            end else if (vme_dtack) begin
                stb_o	<= # dly  1'b1;      
                dat_o	<= # dly vme_data;
            end
		end                             
		s_ack_int: begin
			clear_int		<= # dly 1'b0;
			SendIrqVector	<= # dly 1'b1;	
			if (ds1_shr[1] && vme_iackinn) begin
				state			<= # dly s_idle;
				vme_dtack		<= # dly 1'b1;	
				VmeDOe_o    	<= # dly 1'b0;							
			end else begin
				vme_dtack		<= # dly clear_int; //need to delay 1 clock cycle					
			end
		end
		default: begin
			state	<= # dly s_idle;
		end
	endcase
end
	
endmodule
	
	
	
