module VFCBoard(
    output  [7:1] VmeIrq_ozb7,
    inout   [31:0] VmeD_iozb32,          
    input   VmeSysClk_ik,
    input   VmeAs_in,
    inout   [31:1] VmeA_iozb31,
    input   VmeDtAck_in,
    input   VmeWrite_in,
    input   VmeIack_in,
    input   VmeIackIn_in,
    output  VmeIackOut_on,
    input   [2:1] VmeDs_inb2,
    inout   VmeLword_iozn,
    output  VmeRetry_ozn,
    output  VmeBerr_ozn,
    input   [5:0] VmeAM_ib6, 
    input   VmeSysReset_in, 
    input   VmeTck_i,
    input   VmeTrst_i,
    output  VmeTdo_oz,
    input   VmeGaP_izn, 
    input   [4:0] VmeGa_iznb5, 
    input   VmeTdi_i, 
    input   VmeTms_i,
    
    inout   [4:1] FpGpIo_iob4);

supply0 Gnd;
supply1 Vcc;

wire [7:1] VmeIrq_b7;
wire [31:0] VmeD_b32;          
wire VmeSysClk_k;
wire VmeAs_n;
wire [31:1] VmeA_b31;
wire VmeDtAck_n;
wire VmeWrite_n;
wire VmeIack_n;
wire VmeIackIn_n;
wire VmeIackOut_n;
wire [2:1] VmeDs_nb2;
wire VmeLword_n;
wire VmeRetry_n;
wire VmeBerr_n;
wire [5:0] VmeAM_b6; 
wire VmeSysReset_n; 
wire VmeTck;
wire VmeTrst;
wire VmeTdo;
wire VmeGaP_n; 
wire [4:0] VmeGa_nb5; 
wire VmeTdi; 
wire VmeTms;

//##############################################################
// FRONT PANNEL
//##############################################################

reg i_Pb1_n = 1'b1;
assign (pull1, pull0) a_PushButton_n = i_Pb1_n;

wire [7:0] FpLed_nb8;

pullup  r80(FpLed_nb8[0]),
        r81(FpLed_nb8[1]),
        r82(FpLed_nb8[2]),
        r83(FpLed_nb8[3]),
        r87(FpLed_nb8[4]),
        r88(FpLed_nb8[5]),
        r89(FpLed_nb8[6]),
        r90(FpLed_nb8[7]);

wire [4:1] FpGpIo_b4;
wire FpGpIo1OutputMode, FpGpIo2OutputMode, FpGpIo34OutputMode;

sn74vmeh22501 ic16 (
    .oeab1(FpGpIo1OutputMode),
    .oeby1_n(FpGpIo1OutputMode),
    .a1(FpGpIo_b4[1]),
    .y1(FpGpIo_b4[1]),
    .b1(FpGpIo_iob4[1]),
    .oeab2(FpGpIo2OutputMode),
    .oeby2_n(FpGpIo2OutputMode),
    .a2(FpGpIo_b4[2]),
    .y2(FpGpIo_b4[2]),
    .b2(FpGpIo_iob4[2]),
    .oe_n(Gnd),
    .dir(FpGpIo34OutputMode),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3({FpGpIo_b4[3], Unconnected, Unconnected, Unconnected, FpGpIo_b4[4], Unconnected, Unconnected, Unconnected}),
    .b3({FpGpIo_iob4[3], Unconnected, Unconnected, Unconnected, FpGpIo_iob4[4], Unconnected, Unconnected, Unconnected}));


//##############################################################
// CLKs
//##############################################################

wire OeSi57x_e, SdaSi57x, SclSi57x_k;
wire Si57xClk_kp, Si57xClk_kn;

si57x i_Osc1(
    .oe(OeSi57x_e),
    .sda(SdaSi57x),
    .scl(SclSi57x_k),
    .clk_p(Si57xClk_kp),
    .clk_n(Si57xClk_kn));

ivt3205c25mhz i_Osc2(.out(Vctcxo_k));


//##############################################################
//  VME BUFFERS
//##############################################################

wire Unconnected;

assign (supply1, supply0) Unconnected = 1'bx;

sn74vmeh22501 ic2 (
    .oeab1(VmeIrq_b7[1]),
    .oeby1_n(Vcc),
    .a1(Gnd),
    .y1(Unconnected),
    .b1(VmeIrq_ozb7[1]),
    .oeab2(Gnd),
    .oeby2_n(Gnd),
    .a2(Gnd),
    .y2(VmeIack_n),
    .b2(VmeIack_in),
    .oe_n(VmeDOeN_en),
    .dir(VmeDDir),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3(VmeD_b32[7:0]),
    .b3(VmeD_iozb32[7:0]));

pulldown r64(VmeIrq_b7[1]), 
         r79(VmeDOeN_en);

sn74vmeh22501 ic5 (
    .oeab1(VmeIrq_b7[3]),
    .oeby1_n(Vcc),
    .a1(Gnd),
    .y1(Unconnected),
    .b1(VmeIrq_ozb7[3]),
    .oeab2(VmeIrq_b7[2]),
    .oeby2_n(Vcc),
    .a2(Gnd),
    .y2(Unconnected),
    .b2(VmeIrq_ozb7[2]),
    .oe_n(VmeDOeN_en),
    .dir(VmeDDir),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3(VmeD_b32[15:8]),
    .b3(VmeD_iozb32[15:8]));

pulldown r86(VmeIrq_b7[3]), 
         r91(VmeIrq_b7[2]);

sn74vmeh22501 ic21 (
    .oeab1(Gnd),
    .oeby1_n(Gnd),
    .a1(Gnd),
    .y1(VmeSysClk_k),
    .b1(VmeSysClk_ik),
    .oeab2(Gnd),
    .oeby2_n(Gnd),
    .a2(Gnd),
    .y2(VmeAs_n),
    .b2(VmeAs_in),
    .oe_n(VmeAOeN_en),
    .dir(VmeADir),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3(VmeA_b31[15:8]),
    .b3(VmeA_iozb31[15:8]));

sn74vmeh22501 ic9 (
    .oeab1(VmeDtAckOe_e),
    .oeby1_n(Vcc),
    .a1(VmeDtAck_n),
    .y1(Unconnected),
    .b1(VmeDtAck_in),
    .oeab2(Gnd),
    .oeby2_n(Gnd),
    .a2(Gnd),
    .y2(VmeWrite_n),
    .b2(VmeWrite_in),
    .oe_n(VmeAOeN_en),
    .dir(VmeADir),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3(VmeA_b31[23:16]),
    .b3(VmeA_iozb31[23:16]));

sn74vmeh22501 ic35 (
    .oeab1(VmeIrq_b7[7]),
    .oeby1_n(Vcc),
    .a1(Gnd),
    .y1(Unconnected),
    .b1(VmeIrq_ozb7[7]),
    .oeab2(VmeIrq_b7[6]),
    .oeby2_n(Vcc),
    .a2(Gnd),
    .y2(Unconnected),
    .b2(VmeIrq_ozb7[6]),
    .oe_n(VmeDOeN_en),
    .dir(VmeDDir),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3(VmeD_b32[23:16]),
    .b3(VmeD_iozb32[23:16]));

pulldown r262(VmeIrq_b7[7]), 
         r263(VmeIrq_b7[6]);

sn74vmeh22501 ic42 (
    .oeab1(VmeIrq_b7[5]),
    .oeby1_n(Vcc),
    .a1(Gnd),
    .y1(Unconnected),
    .b1(VmeIrq_ozb7[5]),
    .oeab2(VmeIrq_b7[4]),
    .oeby2_n(Vcc),
    .a2(Gnd),
    .y2(Unconnected),
    .b2(VmeIrq_ozb7[4]),
    .oe_n(VmeDOeN_en),
    .dir(VmeDDir),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3(VmeD_b32[31:24]),
    .b3(VmeD_iozb32[31:24]));

pulldown r279(VmeIrq_b7[5]), 
         r286(VmeIrq_b7[4]);

sn74vmeh22501 ic33 (
    .oeab1(Gnd),
    .oeby1_n(Gnd),
    .a1(Gnd),
    .y1(VmeIackIn_n),
    .b1(VmeIackIn_in),
    .oeab2(Vcc),
    .oeby2_n(Vcc),
    .a2(VmeIackOut_n),
    .y2(Unconnected),
    .b2(VmeIackOut_on),
    .oe_n(VmeAOeN_en),
    .dir(VmeADir),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3(VmeA_b31[31:24]),
    .b3(VmeA_iozb31[31:24]));

assign (pull1, pull0) VmeIackOut_n = VmeIackIn_n; //r54
pullup r239(VmeAOeN_en);

sn74vmeh22501 ic22 (
    .oeab1(Gnd),
    .oeby1_n(Gnd),
    .a1(Gnd),
    .y1(VmeDs_nb2[1]),
    .b1(VmeDs_inb2[1]),
    .oeab2(Gnd),
    .oeby2_n(Gnd),
    .a2(Gnd),
    .y2(VmeDs_nb2[2]),
    .b2(VmeDs_inb2[2]),
    .oe_n(VmeAOeN_en),
    .dir(VmeADir),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3({VmeA_b31[7:1], VmeLword_n}),
    .b3({VmeA_iozb31[7:1], VmeLword_iozn}));

sn74vmeh22501 ic15 (
    .oeab1(VmeRetryOe_e),
    .oeby1_n(Vcc),
    .a1(VmeRetry_n),
    .y1(Unconnected),
    .b1(VmeRetry_ozn),
    .oeab2(VmeBerr),
    .oeby2_n(Vcc),
    .a2(Gnd),
    .y2(Unconnected),
    .b2(VmeBerr_ozn),
    .oe_n(Gnd),
    .dir(Gnd),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3({VmeAM_b6[5:0], VmeSysReset_n, VmeTck}),
    .b3({VmeAM_ib6[5:0], VmeSysReset_in, VmeTck_i}));

pulldown r146(VmeRetryOe_e), 
         r150(VmeBerr);

sn74vmeh22501 ic7 (
    .oeab1(Gnd),
    .oeby1_n(Gnd),
    .a1(Gnd),
    .y1(VmeTrst),
    .b1(VmeTrst_i),
    .oeab2(VmeTdoOe_e),
    .oeby2_n(Vcc),
    .a2(VmeTdo),
    .y2(Unconnected),
    .b2(VmeTdo_oz),
    .oe_n(Gnd),
    .dir(Gnd),
    .clkab(Gnd),
    .le(Vcc),
    .clkba(Gnd),
    .a3({VmeGaP_n, VmeGa_nb5[4:0], VmeTdi, VmeTms}), //inverted GA order to avoid too long concatenations that seem to bother Icarus
    .b3({VmeGaP_izn, VmeGa_iznb5[4:0], VmeTdi_i, VmeTms_i}));

pullup r107(VmeGaP_izn), 
       r108(VmeGa_iznb5[0]), 
       r106(VmeGa_iznb5[1]), 
       r125(VmeGa_iznb5[2]), 
       r124(VmeGa_iznb5[3]), 
       r130(VmeGa_iznb5[4]);


//##############################################################
// SYSTEM FPGA
//##############################################################

reg [8:1] i_Sw1_b8 = 8'hFF;
wire [1:0] a_Switch_b2 = i_Sw1_b8[8:7];
wire a_UseGa = i_Sw1_b8[6];
wire [4:0] a_NoGa_b5 = i_Sw1_b8[5:1];

wire Sys2AppClk, App2SysClk;
wire [2:1] SysAppSlow_b2;
wire [7:0] AFpgaProgD_b8; 
wire AFpgaProgClk;
wire [1:0] AFpgaProgM_b2;
wire AFpgaProgCsi, AFpgaProgRdWr, AFpgaProgInit;  


wire Si57x_k;
assign Si57x_k = Si57xClk_kp;

SystemFpga i_SystemFpga(
    .UseGa_i(a_UseGa),
    .ManualAddress_ib5(a_NoGa_b5), 

    .VmeGa_ib5n(VmeGa_nb5),
    .VmeGaP_in(VmeGaP_n),
    .VmeIrq_ob7(VmeIrq_b7),
    .VmeD_iob32(VmeD_b32),
    .VmeDtAckOe_oe(VmeDtAckOe_e),
    .VmeDtAck_on(VmeDtAck_n),
    .VmeA_iob31(VmeA_b31),
    .VmeLword_io(VmeLword_n),
    .VmeAm_ib6(VmeAM_b6),
    .VmeAs_in(VmeAs_n),
    .VmeSysClk_ik(VmeSysClk_k),
    .VmeWrite_in(VmeWrite_n),
    .VmeDOeN_oen(VmeDOeN_en),
    .VmeDDirVfcToVme_o(VmeDDir),
    .VmeIack_in(VmeIack_n),
    .VmeIackIn_in(VmeIackIn_n),
    .VmeIackOut_on(VmeIackOut_n),
    .VmeDs_inb2(VmeDs_nb2),
    .VmeAOeN_oen(VmeAOeN_en),
    .VmeADirVfcToVme_o(VmeADir),
    .VmeRetryOe_oe(VmeRetryOe_e),
    .VmeRetry_on(VmeRetry_n),
    .VmeBerr_o(VmeBerr),
    .VmeSysReset_in(VmeSysReset_n),
    .VmeTck_i(VmeTck),
    .VmeTrst_i(VmeTrst),
    .VmeTdoOe_oe(VmeTdoOe_e),
    .VmeTdo_o(VmeTdo),
    .VmeTdi_i(VmeTdi),
    .VmeTms_i(VmeTms),

    .VcTcXo_ik(Vctcxo_k),
    .Si57x_ik(Si57x_k), 

    .PushButton_ion(a_PushButton_n),

    .Switch_ib2(a_Switch_b2),

    .FpLed_onb8(FpLed_nb8),
    .FpGpIo1OutputMode_o(FpGpIo1OutputMode),
    .FpGpIo2OutputMode_o(FpGpIo2OutputMode),
    .FpGpIo34OutputMode_o(FpGpIo34OutputMode),

    .SysAppClk_ok(Sys2AppClk),
    .SysAppClk_ik(App2SysClk),
    .SysAppSlow_iob2(SysAppSlow_b2),
    
    .AFpgaProgD_iob8(AFpgaProgD_b8), 
    .AFpgaProgClk_io(AFpgaProgClk),
    .AFpgaProgM_iob2(AFpgaProgM_b2),
    .AFpgaProgCsi_io(AFpgaProgCsi),
    .AFpgaProgRdWr_io(AFpgaProgRdWr),
    .AFpgaProgInit_io(AFpgaProgInit),

    .FlashSFpgaD_o(),
    .FlashSFpgaClk_ok(),
    .FlashSFpgaCs_on(),
    .FlashSFpgaQ_i(),

    .FlashAFpgaD_o(),
    .FlashAFpgaClk_ok(),
    .FlashAFpgaCs_on(),
    .FlashAFpgaQ_i(),

    .VAdcSClk_ok(),
    .VAdcDout_i(),
    .VAdcDin_o(),
    .VAdcCs_on(),

    .VAdjCs_on(VAdjCs),
    .VAdjSClk_ok(VAdjSClk_k),
    .VAdjDin_o(VAdjDin),
    .VAdjInhibit_ozn(),
    .VAdjSpi_o(VAdjSpi),

    .PllFmc1Ld_i(),
    .PllFmc1Status_i(),
    .PllFmc1RefMon_i(),
    .PllFmc1RefSel_o(),
    .PllFmc1Ref1_ok(),
    .PllFmc1Pd_on(),
    .PllFmc1Synch_on(),
    .PllFmc1Reset_orn(),
    .PllFmc1SClk_ok(),
    .PllFmc1SDio_io(),
    .PllFmc1Sdo_i(),
    .PllFmc1Cs_on(),
    .PllFmc12SFpga_ik(),
    
    .PllFmc2Ld_i(),
    .PllFmc2Status_i(),
    .PllFmc2RefMon_i(),
    .PllFmc2RefSel_o(),
    .PllFmc2Ref1_ok(),
    .PllFmc2Pd_on(),
    .PllFmc2Synch_on(),
    .PllFmc2Reset_orn(),
    .PllFmc2SClk_ok(),
    .PllFmc2SDio_io(),
    .PllFmc2Sdo_i(),
    .PllFmc2Cs_on(),
    .PllFmc22SFpga_ik(), 
    
    .PllSysLd_i(),
    .PllSysStatus_i(),
    .PllSysRefMon_i(),
    .PllSysRefSel_o(),
    .PllSysRef12_ok(),
    .PllSysPd_on(),
    .PllSysSynch_on(),
    .PllSysReset_orn(),
    .PllSysSClk_ok(),
    .PllSysSDio_io(),
    .PllSysSdo_i(),
    .PllSysCs_on(),
    .PllSys2SFpga_ik(),
    
    .PllDdsLd_i(),
    .PllDdsStatus_i(),
    .PllDdsRefMon_i(),
    .PllDdsRefSel_o(),
    .PllDdsPd_on(),
    .PllDdsSynch_on(),
    .PllDdsReset_orn(),
    .PllDdsClk_ok(),
    .PllDdsSClk_ok(),
    .PllDdsSDio_io(),
    .PllDdsSdo_i(),
    .PllDdsCs_on(),
    .PllDds2SFpga_ik(),

    .PllDacSClk_ok(),
    .PllDacSynch_on(),
    .PllDacDin_o(),
    .PllDacDout_i(),
    .PllDacClrn_orn(),
    .PllDacLDac_on()   
    );


//##############################################################
// APPLICATION FPGA
//##############################################################
                                                                       
ApplicationFpga i_ApplicationFpga(

    .SysAppClk_ik(Sys2AppClk),
    .SysAppClk_ok(App2SysClk), 
    .SysAppSlow_iob2(SysAppSlow_b2),
    
    .AFpgaProgD_iob8(AFpgaProgD_b8), 
    .AFpgaProgClk_io(AFpgaProgClk),
    .AFpgaProgM_iob2(AFpgaProgM_b2),
    .AFpgaProgCsi_io(AFpgaProgCsi),
    .AFpgaProgRdWr_io(AFpgaProgRdWr),
    .AFpgaProgInit_io(AFpgaProgInit),      


    .PushButton_ion(a_PushButton_n),
    
    .FpGpIo_iob4(FpGpIo_b4)
    );  

//##############################################################
// Power Supplies
//##############################################################  

wire VAdjSpi, VAdjSClk_k, VAdjDin, VAdjCs;

MAX5483 ic44(
    .SPIUD_iq(VAdjSpi),
    .CS_iqn(VAdjCs),
    .SclkInc_iqk(VAdjSClk_k),
    .DinUD_iq(VAdjDin));  

endmodule
