`timescale 1ns/1ns

module tb_VFC ();

supply0 Gnd;
supply1 Vcc;


wire    [6:0] VmeIrq_b7;
wire    [31:0] VmeD_b32;          
wire    VmeSysClk_k;
wire    VmeAs_n;
wire    [31:1] VmeA_b31;
wire    VmeDtAck_n;
wire    VmeWrite_n;
wire    VmeIack_n;
wire    [2:1] VmeDs_nb2;
wire    VmeLword_n;
wire    VmeRetry_n;
wire    VmeBerr_n;
wire    [5:0] VmeAM_b6; 
wire    VmeSysReset_n; 
wire    VmeTck;
wire    VmeTrst;
wire    VmeTdo;
wire    VmeTdi; 
wire    VmeTms;

pullup i_DtAckPU(VmeDtAck_n);
pullup i_RetryPU(VmeRetry_n);
pullup i_BerrPU(VmeBerr_n);
pullup i_IackPU(VmeIack_n);
pullup i_Irq6(VmeIrq_b7[6]);
pullup i_Irq5(VmeIrq_b7[5]);
pullup i_Irq4(VmeIrq_b7[4]);
pullup i_Irq3(VmeIrq_b7[3]);
pullup i_Irq2(VmeIrq_b7[2]);
pullup i_Irq1(VmeIrq_b7[1]);
pullup i_Irq0(VmeIrq_b7[0]);

//####################################
// VME master
//####################################

VmeMaster #(.g_VerboseAccesses(1'b1)) i_VmeMaster(
    .Trst_o(VmeTrst),
    .Tck_o(VmeTck),
    .Tdo_i(VmeTdo),
    .Tdi_o(VmeTdi),
    .Tms_o(VmeTms),
    .D_b(VmeD_b32),
    .SysClk_ok(VmeSysClk_k),
    .Ds1_o(VmeDs_nb2[1]),
    .Ds2_o(VmeDs_nb2[2]),
    .Write_o(VmeWrite_n),
    .DtAck_i(VmeDtAck_n),
    .As_o(VmeAs_n),
    .IAck_o(VmeIack_n), 
    .Am_o(VmeAM_b6),
    .A_b(VmeA_b31),
    .Irq_i(VmeIrq_b7),
    .Retry_i(VmeRetry_n),
    .Berr_i(VmeBerr_n),
    .SysReset_o(VmeSysReset_n),
    .LWord_b(VmeLword_n));

//####################################
// VFC board in SLOT 4
//####################################

wire GapSlot4_n    = ^(5'd4);    
wire [4:0] GaSlot4_nb5 = ~(5'd4);
wire VmeIackInSlot4_n = VmeIack_n;
wire VmeIackOutSlot4_n;
wire [3:0] FpGpIo_b4 = 'hz;

reg WaveformGeneratorOut = 1'b0;

always begin
    #500;
    WaveformGeneratorOut = ~WaveformGeneratorOut;
end

assign FpGpIo_b4[0] =  WaveformGeneratorOut;

VFCBoard i_VFCBoard(
    .VmeIrq_ozb7(VmeIrq_b7),
    .VmeD_iozb32(VmeD_b32),          
    .VmeSysClk_ik(VmeSysClk_k),
    .VmeAs_in(VmeAs_n),
    .VmeA_iozb31(VmeA_b31),
    .VmeDtAck_in(VmeDtAck_n),
    .VmeWrite_in(VmeWrite_n),
    .VmeIack_in(VmeIack_n),
    .VmeIackIn_in(VmeIackInSlot4_n),
    .VmeIackOut_on(VmeIackOutSlot4_n),
    .VmeDs_inb2(VmeDs_nb2),
    .VmeLword_iozn(VmeLword_n),
    .VmeRetry_ozn(VmeRetry_n),
    .VmeBerr_ozn(VmeBerr_n),
    .VmeAM_ib6(VmeAM_b6), 
    .VmeSysReset_in(VmeSysReset_n), 
    .VmeTck_i(VmeTck),
    .VmeTrst_i(VmeTrst),
    .VmeTdo_oz(VmeTdo),
    .VmeGaP_izn(GapSlot4_n), 
    .VmeGa_iznb5(GaSlot4_nb5), 
    .VmeTdi_i(VmeTdi), 
    .VmeTms_i(VmeTms),
    .FpGpIo_iob4(FpGpIo_b4));

endmodule
