from VfcRegistersMap import *
import numpy as np

class VFCInstance(VfcRegisters):
    def __init__(self, Slot, IntLevel, IntVector):
        VfcRegisters.__init__(self, Slot)
        self.IntLevel = IntLevel
        self.IntVector =  IntVector 
    #-----------------------
    # Interrupts
    #-----------------------
    def ConfigureInterrupts(self):
        self.InterruptLvlVect.Write(self.IntLevel*0x10000000+self.IntVector)
    def EnableSpiIdleInterrupt(self):
        self.InterruptConfig.Write(self.InterruptConfig.Read()&0xFFFFFFFD)
    def EnableSpiWaitingDataInterrupt(self):
        self.InterruptConfig.Write(self.InterruptConfig.Read()&0xFFFFFFFE)
    def EnableDebugInterrupt(self):
        self.InterruptConfig.Write(self.InterruptConfig.Read()&0xFFFFFF03)
    def DisableSpiIdleInterrupt(self):
        self.InterruptConfig.Write(self.InterruptConfig.Read()|0x2)
    def DisableSpiWaitingDataInterrupt(self):
        self.InterruptConfig.Write(self.InterruptConfig.Read()|0x1)
    def DisableDebugInterrupt(self):
        self.InterruptConfig.Write(self.InterruptConfig.Read()|0xFC)    
    def GenerateDebugInterrupt(self, Status):
        self.GenericOutputReg1.Write(0x0)
        self.GenericOutputReg1.Write(Status)
        self.GenericOutputReg1.Write(0x0) 
    def WaitInterrupt(self):
        IntStatus = 'X'
        [IntLevel, IntVector] = VmeWaitInterrupt()
        if IntLevel==self.IntLevel :
            if IntVector==self.IntVector : 
                IntStatus = self.InterruptStatus.Read()
            else : print "Wrong Vector: expecting "+ Slot+ " received "+ IntVector
        else : print "Wrong IntLevel: expecting ", self.IntLevel, " received ", IntLevel
        return IntStatus
    #-----------------------
    # SPI 
    #-----------------------
    def SpiAccess(self, Channel, CPol, CPha, LSB1st, Lenght, Data, HalfPeriod, WaitTime):
        if Channel>=32 or Channel<0 :
            print "Error: SPI Channel out of range"
        elif CPol>1 or CPol<0 : 
            print "Error: SPI CPol can be only 0 or 1"
        elif CPha>1 or CPha<0 :
            print "Error: SPI CPha can be only 0 or 1"
        elif LSB1st>1 or LSB1st<0 :
            print "Error: SPI LSB1st can be only 0 or 1"
        elif Lenght>=2**12 or Lenght<0 :
            print "Error: SPI Lenght out of range"
        elif HalfPeriod>=2**16 or HalfPeriod<0 :
            print "Error:SPI  Half period out of range"
        elif WaitTime>=2**16 or WaitTime<0 :
            print "Error: SPI Half period out of range"
        elif Data<0 :
            print "SPI data cannot be negative"
        else :
            SpiStatusReg = self.SpiMasterStatus.Read()
            SpiState = (SpiStatusReg/2**16)&0x7 
            SpiFree = SpiState==4 or SpiState==0
            while SpiFree==False :
                SpiStatusReg = self.SpiMasterStatus.Read()
                SpiState = (SpiStatusReg/2**16)&0x7 
                SpiFree = SpiState==4 or SpiState==0           
            Config1 = CPol*2**31 + CPha*2**30 + LSB1st*2**29 + Channel*2**16 + Lenght 
            Config2 = HalfPeriod*2**16 + WaitTime
            self.SpiMasterConfig1.Write(Config1)
            self.SpiMasterConfig2.Write(Config2)
            Cycles=int(np.ceil(Lenght/32.0))
            ReadValue = 0
            if LSB1st==0 :
                Data = Data*2**(32-Lenght%32)
            for i in range(Cycles):
                if LSB1st :
                    DataToSend = Data%2**32
                    self.SpiMasterShiftOut.Write(DataToSend)
                    Data = Data/2**32
                else:
                    DataToSend = Data/2**(32*(Cycles-(i+1)))
                    self.SpiMasterShiftOut.Write(DataToSend)
                    Data = Data%2**(32*(Cycles-(i+1)))
                SpiStatusReg = self.SpiMasterStatus.Read()
                SpiState = (SpiStatusReg/2**16)&0x7 
                SpiFree = SpiState==4 or SpiState==0
                while SpiFree==False :
                    SpiStatusReg = self.SpiMasterStatus.Read()
                    SpiState = (SpiStatusReg/2**16)&0x7 
                    SpiFree = SpiState==4 or SpiState==0
                PartialShift = self.SpiMasterShiftIn.Read()
                if LSB1st :
                    if i==Cycles-1 :
                        PartialShift = PartialShift/2**((32-Lenght%32)%32)
                    ReadValue = ReadValue + PartialShift*2**(32*i)
                else :
                    if i==Cycles-1 :
                        ReadValue = ReadValue*2**(Lenght%32)+PartialShift
                    else :
                        ReadValue = ReadValue*2**32+PartialShift                    
            return ReadValue
    #-----------------------
    # VAdj Control
    #-----------------------
    def SetVadjWiper(self, Value):
        Data = Value*2**6
        print "Sending value ", hex(Data)
        self.SpiAccess(1, 1, 1, 0, 24, Data, 32, 0)
    def SaveVadjWiper(self):
        self.SpiAccess(1, 1, 1, 0, 8, 0x20, 32, 0) 
    def LoadVadjWiper(self):
        self.SpiAccess(1, 1, 1, 0, 8, 0x30, 32, 0) 
    #-----------------------
    # PLLs Configuration registers
    #-----------------------
#    def ConfPllSpi(self, Pll): 
        #configura la comunicazione scrivendo nel registro 1
#    def WritePllRegister(self, Pll, RegAddress, Value):
        #scivi il valore con davanti la giusta istruzione
        #leggi il valore bufferato
        #se il valore bufferato e' quello che volevamo attiva
        #se il valore non e' buono invia un errore (return error value)
#    def ReadPllBufferedRegister(self, Pll, RegAddress):
        #scivi che vuoi il valore bufferato in reg4
        #leggi il valore del registo
#    def ReadPllActiveRegister(self, Pll, RegAddress):
        #scivi che vuoi il valore attivo in reg4
        #leggi il valore del registo
#    def SetPllRefSel(self, Pll, Value):
        #Leggi il valore, maschera e fai un or
#    def SetPllPd(self, Pll, Value):
        #Leggi il valore, maschera e fai un or
#    def SetPllSynch(self, Pll, Value):
        #Leggi il valore, maschera e fai un or
#    def SetPllReset(self, Pll, Value):
        #Leggi il valore, maschera e fai un or
#    def GetPllsStatus(self):
        #leggi il registro
        #splitta il valore con maschere per ottenere i vari status bit decodificati


